# == Schema Information
#
# Table name: benchmark_configurations
#
#  id                  :integer          not null, primary key
#  name                :string
#  language_id         :integer
#  runtime_id          :integer
#  operating_system_id :integer
#  created_at          :datetime
#  updated_at          :datetime
#





class BenchmarkConfiguration < ActiveRecord::Base

  has_many :benchmark_batches, dependent: :destroy, inverse_of: :benchmark_configuration
  belongs_to :language, inverse_of: :benchmark_configurations
  belongs_to :runtime, inverse_of: :benchmark_configurations
  belongs_to :operating_system, inverse_of: :benchmark_configurations

  def name
    xlated_os = operating_system.name_normalized
    desc = self.description
    if (desc != nil and desc != '')
      return "#{desc} - #{runtime.name} (#{language.name}) on #{xlated_os}"
    else
      return "#{runtime.name} (#{language.name}) on #{xlated_os}"
    end
  end

  def all_benchmark_infos
    result = Set.new
    benchmark_batches.each do |batch|
      batch.benchmark_results.each { |r| result.add r.benchmark_info }
    end
    result
  end

  def all_dates_when_performed
    SortedSet.new(benchmark_batches.map(&:performed_at).flatten).to_a
  end

  def results_at(time)
    batch = benchmark_batches.where(performed_at: time).first
    if batch
      all_benchmark_infos.map do |info|
        batch.result_for(info)
      end
    else
      []
    end
  end

  def results_of(benchmark_info, time)
    batch_with_given_date = benchmark_batches
    .where(performed_at: time)
    .first

    if batch_with_given_date
      batch_with_given_date
      .benchmark_results(benchmark_info: benchmark_info)
      .first
    else
      nil
    end
  end

  def results_for_info(info)
    results = benchmark_batches.collect do |batch|
      batch.result_for(info)
    end
    results.delete_if(&:nil?)
  end

  def latest_batch_for(info)
    results_for_info(info).max { |a, b| a.performed_at <=> b.performed_at }
  end

  def latest_result_for(info)
    latest_batch_for(info).try(:min_duration)
  end

  def latest_parameters_for(info)
    latest_batch_for(info).try(:latest_results_for, info).try(:benchmark_parameters_json) || []
  end

  def to_json()
    all_benchmark_infos.collect do |info|
      {
        name: info.name,
        data: (results_for_info(info).select { | r | r != nil }).collect { |r| r.to_json }
      }
    end
  end

  def BenchmarkConfiguration.from_json(data)
    lang = find_or_create_by_name(Language, data["language"])
    runtime = find_or_create_by_name(lang.runtimes, data["runtime"])
    os = find_or_create_by_name(OperatingSystem, data["os"])
    desc = data["description"] || ""

    configuration_params = {language: lang, runtime: runtime, operating_system: os, description: desc }
    configuration = BenchmarkConfiguration.where(configuration_params).first
    configuration = BenchmarkConfiguration.create!(configuration_params) unless configuration
    return configuration
  end
end

def find_or_create_by_name(ctx, name)
    inst = ctx.where(name: name).first
    inst = ctx.create!(name: name) unless inst
    inst
end
